@@ -24,19 +24,19 @@ BEGIN {
         die $@ if $@;
     }
 
-    # Set up _Debug_SQL if requested
+    # Set up DebugSQL if requested
     require DBIx::DBO::Common;
 
     if ($ENV{DBO_DEBUG_SQL}) {
         diag "DBO_DEBUG_SQL=$ENV{DBO_DEBUG_SQL}";
         package DBIx::DBO::Common;
 
-        $DBIx::DBO::Common::Config{_Debug_SQL} = $ENV{DBO_DEBUG_SQL};
+        $DBIx::DBO::Common::Config{DebugSQL} = $ENV{DBO_DEBUG_SQL};
         no warnings 'redefine';
         *DBIx::DBO::Common::_carp_last_sql = sub {
             my $me = shift;
             my ($cmd, $sql, @bind) = @{$me->_last_sql};
-            local $Carp::Verbose = 1 if $me->config('_Debug_SQL') > 1;
+            local $Carp::Verbose = 1 if $me->config('DebugSQL') > 1;
             my @mess = split /\n/, Carp::shortmess("\t$cmd called");
             splice @mess, 0, 3 if $Carp::Verbose;
             Test::More::diag join "\n", "DEBUG_SQL: $sql", 'DEBUG_SQL: ('.join(', ', map $me->rdbh->quote($_), @bind).')', @mess;
@@ -47,7 +47,9 @@ use DBIx::DBO;
 
 our $dbd;
 our $dbd_name;
-(our $prefix = "DBO_${DBIx::DBO::VERSION}_test") =~ s/\W/_/g;
+(our $test_db = "DBO_${DBIx::DBO::VERSION}_test_db") =~ s/\W/_/g;
+(our $test_sch = "DBO_${DBIx::DBO::VERSION}_test_sch") =~ s/\W/_/g;
+(our $test_tbl = "DBO_${DBIx::DBO::VERSION}_test_tbl") =~ s/\W/_/g;
 our @_cleanup_sql;
 
 sub import {
@@ -92,15 +94,18 @@ sub import {
         }
     }
 
+    if (exists $opt{try_connect}) {
+        try_to_connect($opt{try_connect});
+    }
+
     return unless exists $opt{tests};
 
     if (exists $opt{connect_ok}) {
-        my $dbo_ref = shift @{$opt{connect_ok}};
-        $$dbo_ref = connect_dbo(@{$opt{connect_ok}}) or plan skip_all => "Can't connect: $DBI::errstr";
+        my $dbo = connect_ok(@{$opt{connect_ok}}) or plan skip_all => "Can't connect: $DBI::errstr";
 
         plan tests => $opt{tests};
         pass "Connect to $dbd_name";
-        isa_ok $$dbo_ref, "DBIx::DBO::DBD::${dbd}", '$dbo';
+        isa_ok $dbo, "DBIx::DBO::DBD::${dbd}", '$dbo';
     } else {
         plan tests => $opt{tests};
     }
@@ -119,29 +124,35 @@ sub sql_err {
     join "\n", @err;
 }
 
-sub connect_ok {
-    ok my $dbo = connect_dbo(@_), "Connect to $dbd_name" or die $DBI::errstr;
-    isa_ok $dbo, "DBIx::DBO::DBD::${dbd}", '$dbo';
-    return $dbo;
+sub last_sql {
+    my $me = shift;
+    my ($cmd, $sql, @bind) = @{$me->_last_sql};
+    print $sql, "\n(".join(', ', map $me->rdbh->quote($_), @bind).")\n";
 }
 
 sub connect_dbo {
-    my $dsn = shift;
-    defined $dsn or $dsn = $ENV{'DBO_TEST_'.uc($dbd).'_DB'};
+    my ($dsn, $user, $pass) = @_;
     defined $dsn or $dsn = '';
-    my $user = shift;
-    defined $user or $user = $ENV{'DBO_TEST_'.uc($dbd).'_USER'};
-    my $pass = shift;
-    defined $pass or $pass = $ENV{'DBO_TEST_'.uc($dbd).'_PASS'};
-
     DBIx::DBO->connect("DBI:$dbd:$dsn", $user, $pass, {RaiseError => 0});
 }
 
+sub try_to_connect {
+    my $dbo_ref = shift;
+    my @env = map $ENV{"DBO_TEST_\U$dbd\E_$_"}, qw(DSN USER PASS);
+    if (grep defined, @env) {
+        return $$dbo_ref = connect_dbo(@env);
+    }
+    return undef;
+}
+
+sub connect_ok {
+    my $dbo_ref = shift;
+    return try_to_connect($dbo_ref) || ($$dbo_ref = connect_dbo(@_));
+}
+
 sub basic_methods {
     my $dbo = shift;
-    my $schema = shift;
-    my $table = shift;
-    my $quoted_table = $dbo->_qi($schema, $table);
+    my $quoted_table = $dbo->_qi($test_sch, $test_tbl);
     my $t;
 
     # Create a test table with a multi-column primary key
@@ -149,7 +160,7 @@ sub basic_methods {
         pass 'Create a test table';
 
         # Create a table object
-        $t = $dbo->table([$schema, $table]);
+        $t = $dbo->table([$test_sch, $test_tbl]);
         isa_ok $t, 'DBIx::DBO::Table', '$t';
 
         # Check the Primary Keys
@@ -159,7 +170,7 @@ sub basic_methods {
         $dbo->do("DROP TABLE $quoted_table") && $dbo->do("CREATE TABLE $quoted_table (id INT, name TEXT)")
             or diag sql_err($dbo) or die "Can't recreate the test table!\n";
         $dbo->_get_table_info($t->{Schema}, $t->{Name});
-        $t = $dbo->table([$schema, $table]);
+        $t = $dbo->table([$test_sch, $test_tbl]);
     }
     else {
         diag sql_err($dbo);
@@ -172,9 +183,11 @@ sub basic_methods {
             or diag sql_err($dbo) or die "Can't create the test table!\n";
 
         # Create our table object
-        $t = $dbo->table([$schema, $table]);
+        $t = $dbo->table([$test_sch, $test_tbl]);
         isa_ok $t, 'DBIx::DBO::Table', '$t';
     }
+    die "Couldn't create the DBIx::DBO::Table object!" unless $t;
+
     pass 'Method DBIx::DBO->do';
 
     # Insert data
@@ -222,7 +235,7 @@ sub basic_methods {
     is $rv, 1, 'Method DBIx::DBO::Table->delete';
 
     # Remove the created table during cleanup
-    push @_cleanup_sql, "DROP TABLE $quoted_table";
+    todo_cleanup("DROP TABLE $quoted_table");
 
     return $t;
 }
@@ -334,12 +347,20 @@ sub query_methods {
     1 while $q->fetch;
     is $q->rows, 6, 'Row count is 6';
 
+    # Parentheses
+    $q->where('name', 'LIKE', \"'%o%'");
+    $q->open_bracket('OR');
+    $q->where('name', 'LIKE', \"'%a%'");
+    $q->where('id', '!=', \1);
+    $q->close_bracket;
+    is_deeply $q->col_arrayref({ Columns => [1] }), [4,5,6], 'Method DBIx::DBO::Query->open_bracket';
+
     # Reset the Query
     $q->reset;
     is $q->sql, $dbo->query($t)->sql, 'Method DBIx::DBO::Query->reset';
 
     # Group by the first initial
-    $q->show({FUNC => 'SUBSTRING(?, 1, 1)', COL => 'name', AS => 'initial'});
+    $q->show({FUNC => 'SUBSTR(?, 1, 1)', COL => 'name', AS => 'initial'});
     ok(($q->group_by('initial'), $q->run), 'Method DBIx::DBO::Query->group_by') or diag sql_err($q);
 
     $q->finish;
@@ -434,6 +455,11 @@ sub join_methods {
     $q->finish;
 }
 
+sub todo_cleanup {
+    my $sql = shift;
+    unshift @_cleanup_sql, $sql;
+}
+
 sub cleanup {
     my $dbo = shift;
 
@@ -441,6 +467,9 @@ sub cleanup {
     for my $sql (@_cleanup_sql) {
         $dbo->do($sql) or diag sql_err($dbo);
     }
+
+    $dbo->disconnect;
+    ok !defined $dbo->{dbh} && !defined $dbo->{rdbh}, 'Method DBIx::DBO->disconnect';
 }
 
 my @_no_recursion;
@@ -469,7 +498,7 @@ sub Dump {
         elsif (reftype $val eq 'REF')  { _Find_Seen(\%seen, $$val) }
         $d->Seen(\%seen);
     }
-    warn $d->Dump;
+    print $d->Dump;
 }
 
 sub _Find_Seen {
