@@ -17,7 +17,7 @@ use subs qw(ouch oops);
 
 our %Config = (
     QuoteIdentifier => 1,
-    _Debug_SQL => 0,
+    DebugSQL => 0,
 );
 our @CARP_NOT;
 our $placeholder = PLACEHOLDER;
@@ -55,7 +55,7 @@ sub _last_sql {
 sub _carp_last_sql {
     my $me = shift;
     my ($cmd, $sql, @bind) = @{$me->_last_sql};
-    local $Carp::Verbose = 1 if $me->config('_Debug_SQL') > 1;
+    local $Carp::Verbose = 1 if $me->config('DebugSQL') > 1;
     my @mess = split /\n/, Carp::shortmess("\t$cmd called");
     splice @mess, 0, 3 if $Carp::Verbose;
     warn join "\n", $sql, '('.join(', ', map $me->rdbh->quote($_), @bind).')', @mess;
@@ -65,7 +65,7 @@ sub _sql {
     my ($me, $sql) = splice @_, 0, 2;
     my $cmd = (caller(1))[3];
     $me->_last_sql($cmd, $sql, @_);
-    $me->_carp_last_sql if $me->config('_Debug_SQL');
+    $me->_carp_last_sql if $me->config('DebugSQL');
 }
 
 sub do {
@@ -163,8 +163,8 @@ sub _build_from {
     $h->{from} = $me->_build_table(($me->tables)[0]);
     for (my $i = 1; $i < $me->tables; $i++) {
         $h->{from} .= $h->{Join}[$i].$me->_build_table(($me->tables)[$i]);
-        if ($h->{JoinOn}[$i]) {
-            $h->{from} .= ' ON '.join(' AND ', $me->_build_where_chunk($h->{From_Bind}, 'OR', $h->{JoinOn}[$i]));
+        if ($h->{Join_On}[$i]) {
+            $h->{from} .= ' ON '.join(' AND ', $me->_build_where_chunk($h->{From_Bind}, 'OR', $h->{Join_On}[$i]));
         }
     }
     $h->{from};
@@ -281,6 +281,7 @@ sub _build_val {
     return $func.$extra;
 }
 
+# Construct the WHERE clause
 sub _build_where {
     my $me = shift;
     my $h = shift;
@@ -292,6 +293,7 @@ sub _build_where {
     $h->{where} = join ' AND ', @where;
 }
 
+# Construct the WHERE contents of one set of parentheses
 sub _build_where_chunk {
     my $me = shift;
     my ($bind, $ag, $whs) = @_;
@@ -332,11 +334,13 @@ sub _op_ag {
     return 'AND' if $_[0] eq '!=' or $_[0] eq 'IS NOT' or $_[0] eq '<>' or $_[0] eq 'NOT IN' or $_[0] eq 'NOT BETWEEN';
 }
 
+# Construct one WHERE expression
 sub _build_where_piece {
     my ($me, $bind, $op, $fld, $fld_func, $fld_opt, $val, $val_func, $val_opt) = @_;
     $me->_build_val($bind, $fld, $fld_func, $fld_opt)." $op ".$me->_build_val($bind, $val, $val_func, $val_opt);
 }
 
+# Construct one WHERE expression (simple)
 sub _build_quick_where {
     ouch 'Wrong number of arguments' if @_ & 1;
     my ($me, $bind) = splice @_, 0, 2;
@@ -389,4 +393,14 @@ sub _build_limit {
     $h->{limit};
 }
 
+sub _set_config {
+    my $me = shift;
+    my ($ref, $opt, $val) = @_;
+    ouch "Invalid value for the 'UseHandle' setting"
+        if $opt eq 'UseHandle' and defined $val and $val ne 'read-only' and $val ne 'read-write';
+    my $old = $ref->{$opt};
+    $ref->{$opt} = $val;
+    return $old;
+}
+
 1;
