@@ -34,33 +34,35 @@ DBIx::DBO::Query - An OO interface to SQL queries and results.  Encapsulates an
   $query->where($table1 ** 'age', '<', 20, FORCE => 'OR'); # Force OR so that we get: (age < 20 OR age > 30)
   $query->where($table1 ** 'age', '>', 30, FORCE => 'OR'); # instead of the default: (age < 20 AND age > 30)
 
+=head1 DESCRIPTION
+
+A C<Query> object represents rows from a database (from one or more tables). This module makes it easy, not only to fetch and use the data in the returned rows, but also to modify the query to return a different result set.
+
+=head1 SUBCLASSING
+
+When subclassing C<DBIx::DBO::Query>, please note that C<Query> objects created with the L</new> method are blessed into a DBD driver specific module. For example if using MySQL, a new C<Query> object will be blessed into C<DBIx::DBO::Query::DBD::mysql> which inherits from C<DBIx::DBO::Query>.  However if objects are created from a subclass called C<MySubClass> the new object will be blessed into C<MySubClass::DBD::mysql> which will inherit from both C<MySubClass> and C<DBIx::DBO::Query::DBD::mysql>.
+
+For this reason it's advisable that L<MRO::Compat|MRO::Compat> is installed if the perl version you are using is less than 5.9.5 as this module will try to ensure that the 'C3' method resolution order is used.
+
 =head1 METHODS
 
-=head2 dbh
+=head3 C<new>
 
-The read-write C<DBI> handle.
+  DBIx::DBO::Query->new($dbo, $table1, ...);
 
-=head2 rdbh
-
-The read-only C<DBI> handle, or if there is no read-only connection, the read-write C<DBI> handle.
-
-=head2 do
-
-  $dbo->do($statement)         or die $dbo->dbh->errstr;
-  $dbo->do($statement, \%attr) or die $dbo->dbh->errstr;
-  $dbo->do($statement, \%attr, @bind_values) or die ...
-
-This provides access to L<DBI-E<gt>do|DBI/"do"> method. It defaults to using the read-write C<DBI> handle.
+Create a new C<Query> object from the tables specified.
+In scalar context, just the C<Query> object will be returned.
+In list context, the C<Query> object and L<DBIx::DBO::Table|DBIx::DBO::Table> objects will be returned for each table specified.
 
 =cut
 
-sub _new {
+sub new {
     my $proto = shift;
     my $class = ref($proto) || $proto;
     my $me = { DBO => shift, sql => undef };
     blessed $me->{DBO} and $me->{DBO}->isa('DBIx::DBO') or ouch 'Invalid DBO Object';
     ouch 'No table specified in new Query' unless @_;
-    bless $me, $class;
+    bless $me, $me->{DBO}->_create_dbd_class($class, __PACKAGE__);
 
     for my $table (@_) {
         $me->join_table($table);
@@ -69,46 +71,27 @@ sub _new {
     return wantarray ? ($me, $me->tables) : $me;
 }
 
-=head2 join_table
+=head3 C<reset>
 
-  $query->join_table($table, $join_type);
-  $query->join_table([$schema, $table], $join_type);
-  $query->join_table($table_object, $join_type);
+  $query->reset;
 
-Join a table onto the query, creating a L<DBIx::DBO::Table|DBIx::DBO::Table> object if needed.
-This will perform a comma (", ") join unless $join_type is specified.
+Reset the query, start over with a clean slate.
 
-Returns the C<Table> object.
+B<NB>: This will not remove the JOINs or JOIN ON clauses.
 
 =cut
 
-##
-# Comma, INNER, NATURAL, LEFT, RIGHT, FULL
-##
-sub join_table {
-    my ($me, $tbl, $type) = @_;
-    if (blessed $tbl and $tbl->isa('DBIx::DBO::Table')) {
-        ouch 'This table is already in this query' if $me->_table_idx($tbl);
-    } else {
-        $tbl = $me->{DBO}->table($tbl);
-    }
-    if (defined $type) {
-        $type =~ s/^\s*/ /;
-        $type =~ s/\s*$/ /;
-        $type = uc $type;
-        $type .= 'JOIN ' if $type !~ /\bJOIN\b/;
-    } else {
-        $type = ', ';
-    }
-    push @{$me->{Tables}}, $tbl;
-    push @{$me->{build_data}{Join}}, $type;
-    push @{$me->{build_data}{JoinOn}}, undef;
-    undef $me->{sql};
-    undef $me->{build_data}{from};
-    return $tbl;
+sub reset {
+    my $me = shift;
+    $me->unwhere;
+#    $me->{IsDistinct} = 0;
+    $me->show;
+    $me->group_by;
+    $me->order_by;
+    $me->limit;
 }
 
-=head2 tables
+=head3 C<tables>
 
 Return a list of L<DBIx::DBO::Table|DBIx::DBO::Table> objects for this query.
 
@@ -134,30 +117,12 @@ sub _table_alias {
     @{$me->{Tables}} > 1 ? 't'.($i + 1) : ();
 }
 
-=head2 reset
-
-  $query->reset;
-
-Reset the query, start over with a clean slate.
-
-=cut
-
-sub reset {
-    my $me = shift;
-    $me->unwhere;
-#    $me->{IsDistinct} = 0;
-    $me->show;
-    $me->group_by;
-    $me->order_by;
-    $me->limit;
-}
-
-=head2 show
+=head3 C<show>
 
   $query->show(@columns);
   $query->show($table1 ** 'id', {FUNC => 'UCASE(?)', COL => 'name', AS => 'NAME'}, ...
 
-Specify which columns to show as an array. If the array is empty all columns will be shown.
+Specify which columns to show as an array.  If the array is empty all columns will be shown.
 
 =cut
 
@@ -177,12 +142,55 @@ sub show {
     }
 }
 
-=head2 join_on
+=head3 C<join_table>
+
+  $query->join_table($table, $join_type);
+  $query->join_table([$schema, $table], $join_type);
+  $query->join_table($table_object, $join_type);
+
+Join a table onto the query, creating a L<DBIx::DBO::Table|DBIx::DBO::Table> object if needed.
+This will perform a comma (", ") join unless $join_type is specified.
+
+Valid join types are any accepted by the DB.  Eg: C<'JOIN'>, C<'LEFT'>, C<'RIGHT'>, C<undef> (for comma join), C<'INNER'>, C<'OUTER'>, ...
+
+Returns the C<Table> object.
+
+=cut
+
+##
+# Comma, INNER, NATURAL, LEFT, RIGHT, FULL
+##
+sub join_table {
+    my ($me, $tbl, $type) = @_;
+    if (blessed $tbl and $tbl->isa('DBIx::DBO::Table')) {
+        ouch 'This table is already in this query' if $me->_table_idx($tbl);
+    } else {
+        $tbl = $me->{DBO}->table($tbl);
+    }
+    if (defined $type) {
+        $type =~ s/^\s*/ /;
+        $type =~ s/\s*$/ /;
+        $type = uc $type;
+        $type .= 'JOIN ' if $type !~ /\bJOIN\b/;
+    } else {
+        $type = ', ';
+    }
+    push @{$me->{Tables}}, $tbl;
+    push @{$me->{build_data}{Join}}, $type;
+    push @{$me->{build_data}{Join_On}}, undef;
+#    push @{$me->{Join_Bracket_Refs}}, [];
+#    push @{$me->{Join_Brackets}}, [];
+    undef $me->{sql};
+    undef $me->{build_data}{from};
+    return $tbl;
+}
+
+=head3 C<join_on>
 
   $query->join_on($table_object, $expression1, $operator, $expression2);
   $query->join_on($table2, $table1 ** 'id', '=', $table2 ** 'id');
 
-Join tables on a specific WHERE clause. The first argument is the table object being joined onto.
+Join tables on a specific WHERE clause.  The first argument is the table object being joined onto.
 Then a JOIN ON condition follows, which uses the same arguments as L</where>.
 
 =cut
@@ -210,11 +218,34 @@ sub join_on {
     undef $me->{build_data}{from};
 
     $me->{build_data}{Join}[$i] = ' JOIN ' if $me->{build_data}{Join}[$i] eq ', ';
-    $me->_add_where($me->{build_data}{JoinOn}[$i] ||= [], $op,
+    $me->_add_where($me->{build_data}{Join_On}[$i] ||= [], $op,
         $col1, $col1_func, $col1_opt, $col2, $col2_func, $col2_opt, @_);
 }
 
-=head2 where
+#=head3 C<open_join_on_bracket>, C<close_join_on_bracket>
+#
+#  $query->open_join_on_bracket($table, 'OR');
+#  $query->join_on(...
+#  $query->close_join_on_bracket($table);
+#
+#Equivalent to L<open_bracket|/open_bracket__close_bracket>, but for the JOIN ON clause.
+#The first argument is the table being joined onto.
+#
+#=cut
+#
+#sub open_join_on_bracket {
+#    my $me = shift;
+#    my $i = $me->_table_idx(shift) or ouch 'No such table object in the join';
+#    $me->_open_bracket($me->{Join_Brackets}[$i], $me->{Join_Bracket_Refs}[$i], $me->{build_data}{Join_On}[$i] ||= [], @_);
+#}
+#
+#sub close_join_on_bracket {
+#    my $me = shift;
+#    my $i = $me->_table_idx(shift) or ouch 'No such table object in the join';
+#    $me->_close_bracket($me->{Join_Brackets}[$i], $me->{Join_Bracket_Refs}[$i]);
+#}
+
+=head3 C<where>
 
 Restrict the query with the condition specified (WHERE clause).
 
@@ -223,27 +254,61 @@ Restrict the query with the condition specified (WHERE clause).
 
 C<$operator> is one of: C<'=', '<', 'E<gt>', 'IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN', ...>
 
-C<$expression> can be any of the following:
+C<$expression>s can be any of the following:
 
-    A SCALAR value: C<123> or C<'hello'>
-    A SCALAR reference: C<\"22 * 3">  (These are passed unqouted in the SQL statement!)
-    An ARRAY reference: C<[1, 3, 5]>  (Used with C<IN> and C<BETWEEN> etc)
-    A Table Column: C<$table ** 'id'> or C<$table->column('id')>
-    A Hash reference: (Described below)
+=over 4
 
-For a more complex where clause the expression can be passed as a hash reference.
-Possibly containing SCALARs, ARRAYs or Table Columns.
+=item *
+A scalar value: C<123> or C<'hello'> (or for C<$expression1> a column name: C<'id'>)
+
+=item *
+A scalar reference: C<\"22 * 3">  (These are passed unquoted in the SQL statement!)
+
+=item *
+An array reference: C<[1, 3, 5]>  (Used with C<IN> and C<BETWEEN> etc)
+
+=item *
+A Column object: C<$table ** 'id'> or C<$table->column('id')>
+
+=item *
+A hash reference: (Described below)
+
+=back
+
+For a more complex where expression it can be passed as a hash reference.
+Possibly containing scalars, arrays or Column objects.
 
   $query->where('name', '=', { FUNC => 'COALESCE(?,?)', VAL => [$name, 'Unknown'] });
   $query->where('string', '=', { FUNC => "CONCAT('Mr. ',?)", COL => 'name' });
 
 The keys to the hash in a complex expression are:
 
-  VAL       A scalar, scalar reference or an array reference.
-  COL       The name of a column or a Column object.
-  AS        An alias name.
-  FUNC      A string to be inseted into the SQL, possibly containing "?" placeholders.
-  ORDER     To order by a column.
+=over 4
+
+=item *
+C<VAL> => A scalar, scalar reference or an array reference.
+
+=item *
+C<COL> => The name of a column or a Column object.
+
+=item *
+C<AS> => An alias name.
+
+=item *
+C<FUNC> => A string to be inserted into the SQL, possibly containing "?" placeholders.
+
+=item *
+C<ORDER> => To order by a column (Used only in C<group_by> and C<order_by>).
+
+=back
+
+Multiple C<where> expressions are combined I<cleverly> using the preferred aggregator C<'AND'> (unless L<open_bracket|/open_bracket__close_bracket> was used to change this).  So that when you add where expressions to the query, they will be C<AND>ed together.  However some expressions that refer to the same caloumn will automatically be C<OR>ed instead where this makes sense, currently: C<'='>, C<'IS NULL'>, C<E<lt>=E<gt>>, C<IN> and C<'BETWEEN'>.  Similarly, when the preferred aggregator is C<'OR'> the following operators will be C<AND>ed together: C<'!='>, C<'IS NOT NULL'>, C<E<lt>E<gt>>, C<NOT IN> and C<'NOT BETWEEN'>.
+
+  $query->where('id', '=', 5);
+  $query->where('name', '=', 'Bob');
+  $query->where('id', '=', 7);
+  $query->where(...
+  # Produces: WHERE ("id" = 5 OR "id" = 7) AND "name" = 'Bob' AND ...
 
 =cut
 
@@ -270,16 +335,15 @@ sub where {
 
     # Find the current Where_Data reference
     my $ref = $me->{build_data}{Where_Data} ||= [];
-    $ref = $ref->[$_] for (@{$me->{Bracket_Refs}});
+    $ref = $ref->[$_] for (@{$me->{Where_Bracket_Refs}});
 
     $me->_add_where($ref, $op, $fld, $fld_func, $fld_opt, $val, $val_func, $val_opt, @_);
 }
 
-=head2 unwhere
+=head3 C<unwhere>
 
   $query->unwhere();
   $query->unwhere($column);
-  $query->unwhere($table1 ** 'id');
 
 Removes all previously added where() restrictions for a column.
 If no column is provided, ALL where() restrictions are removed.
@@ -295,7 +359,7 @@ sub unwhere {
         if (exists $me->{build_data}{Where_Data}) {
             # Find the current Where_Data reference
             my $ref = $me->{build_data}{Where_Data};
-            $ref = $ref->[$_] for (@{$me->{Bracket_Refs}});
+            $ref = $ref->[$_] for (@{$me->{Where_Bracket_Refs}});
 
             for (my $i = $#$ref; $i >= 0; $i--) {
                 # Remove this Where piece if there is no FUNC and it refers to this column only
@@ -305,8 +369,8 @@ sub unwhere {
         }
     } else {
         delete $me->{build_data}{Where_Data};
-        $me->{Bracket_Refs} = [];
-        $me->{Brackets} = [];
+        $me->{Where_Bracket_Refs} = [];
+        $me->{Where_Brackets} = [];
     }
     # This forces a new search
     undef $me->{sql};
@@ -318,7 +382,7 @@ sub unwhere {
 # The arrayref will contain 5 values:
 #  $op, $fld_func, $fld, $val_func, $val, $force
 #  $op is the operator (those supported differ by DBD)
-#  $fld_func is undef or a SCALAR of the form '? AND ?' or 'POSITION(? IN ?)'
+#  $fld_func is undef or a scalar of the form '? AND ?' or 'POSITION(? IN ?)'
 #  $fld is an arrayref of columns/values for use with $fld_func
 #  $val_func is similar to $fld_func
 #  $val is an arrayref of values for use with $val_func
@@ -349,7 +413,7 @@ sub _add_where {
             } else {
                 $val = [ $val ];
             }
-            # Add to previous 'IN' and 'NOT IN' Wheres
+            # Add to previous 'IN' and 'NOT IN' Where expressions
             unless ($opt{FORCE} and $opt{FORCE} ne _op_ag($op)) {
                 for my $lim (grep $$_[0] eq $op, @{$ref}) {
                     next if defined $$lim[1] xor defined $fld;
@@ -387,13 +451,59 @@ sub _parse_col_val {
     return [ $me->_parse_col($col, $c{Aliases}) ];
 }
 
-=head2 group_by
+=head3 C<open_bracket>, C<close_bracket>
+
+  $query->open_bracket('OR');
+  $query->where( ...
+  $query->where( ...
+  $query->close_bracket;
+
+Used to group C<where> expressions together in parenthesis using either C<'AND'> or C<'OR'> as the preferred aggregator.
+All the C<where> calls made between C<open_bracket> and C<close_bracket> will be inside the parenthesis.
+
+Without any parenthesis C<'AND'> is the preferred aggregator.
+
+=cut
+
+sub open_bracket {
+    my $me = shift;
+    $me->_open_bracket($me->{Where_Brackets}, $me->{Where_Bracket_Refs}, $me->{build_data}{Where_Data} ||= [], @_);
+}
+
+sub _open_bracket {
+    my ($me, $brackets, $bracket_refs, $ref, $ag) = @_;
+    ouch 'Invalid argument MUST be AND or OR' if !$ag or $ag !~ /^(AND|OR)$/;
+    my $last = @$brackets ? $brackets->[-1] : 'AND';
+    if ($ag ne $last) {
+        # Find the current data reference
+        $ref = $ref->[$_] for @$bracket_refs;
+
+        push @$ref, [];
+        push @$bracket_refs, $#$ref;
+    }
+    push @$brackets, $ag;
+}
+
+sub close_bracket {
+    my $me = shift;
+    $me->_close_bracket($me->{Where_Brackets}, $me->{Where_Bracket_Refs});
+}
+
+sub _close_bracket {
+    my ($me, $brackets, $bracket_refs) = @_;
+    my $ag = pop @{$brackets} or ouch "Can't close bracket with no open bracket!";
+    my $last = @$brackets ? $brackets->[-1] : 'AND';
+    pop @$bracket_refs if $last ne $ag;
+    return $ag;
+}
+
+=head3 C<group_by>
 
   $query->group_by('column', ...);
   $query->group_by($table ** 'column', ...);
   $query->group_by({ COL => $table ** 'column', ORDER => 'DESC' }, ...);
 
-Group the results by the column(s) listed. This will replace the GROUP BY clause.
+Group the results by the column(s) listed.  This will replace the GROUP BY clause.
 To remove the GROUP BY clause simply call C<group_by> without any columns.
 
 =cut
@@ -409,13 +519,13 @@ sub group_by {
     }
 }
 
-=head2 order_by
+=head3 C<order_by>
 
   $query->order_by('column', ...);
   $query->order_by($table ** 'column', ...);
   $query->order_by({ COL => $table ** 'column', ORDER => 'DESC' }, ...);
 
-Order the results by the column(s) listed. This will replace the ORDER BY clause.
+Order the results by the column(s) listed.  This will replace the ORDER BY clause.
 To remove the ORDER BY clause simply call C<order_by> without any columns.
 
 =cut
@@ -431,7 +541,7 @@ sub order_by {
     }
 }
 
-=head2 limit
+=head3 C<limit>
 
   $query->limit;
   $query->limit($rows);
@@ -451,7 +561,7 @@ sub limit {
     @{$me->{build_data}{LimitOffset}} = ($rows, $offset);
 }
 
-=head2 arrayref
+=head3 C<arrayref>
 
   $query->arrayref;
   $query->arrayref(\%attr);
@@ -465,11 +575,10 @@ sub arrayref {
     my $me = shift;
     my $attr = shift;
     $me->_sql($me->sql, $me->_bind_params_select($me->{build_data}));
-    my $sql_or_sth = $me->sth->{Active} ? 'sql' : 'sth';
-    $me->rdbh->selectall_arrayref($me->{$sql_or_sth}, $attr, $me->_bind_params_select($me->{build_data}));
+    $me->rdbh->selectall_arrayref($me->{sql}, $attr, $me->_bind_params_select($me->{build_data}));
 }
 
-=head2 hashref
+=head3 C<hashref>
 
   $query->hashref($key_field);
   $query->hashref($key_field, \%attr);
@@ -484,16 +593,15 @@ sub hashref {
     my $key = shift;
     my $attr = shift;
     $me->_sql($me->sql, $me->_bind_params_select($me->{build_data}));
-    my $sql_or_sth = $me->sth->{Active} ? 'sql' : 'sth';
-    $me->rdbh->selectall_hashref($me->{$sql_or_sth}, $key, $attr, $me->_bind_params_select($me->{build_data}));
+    $me->rdbh->selectall_hashref($me->{sql}, $key, $attr, $me->_bind_params_select($me->{build_data}));
 }
 
-=head2 col_arrayref
+=head3 C<col_arrayref>
 
   $query->col_arrayref;
   $query->col_arrayref(\%attr);
 
-Run the query using L<DBI-E<gt>selectcol_arrayref|DBI/"selectcol_arrayref"> which returns the result as an arrayref of the values of each row in one array. By default it pushes all the columns requested by the L</show> method onto the result array (this differs from the C<DBI>). Or to specify which columns to include in the result use the 'Columns' attribute in C<%attr> - see L<DBI-E<gt>selectcol_arrayref|DBI/"selectcol_arrayref">.
+Run the query using L<DBI-E<gt>selectcol_arrayref|DBI/"selectcol_arrayref"> which returns the result as an arrayref of the values of each row in one array.  By default it pushes all the columns requested by the L</show> method onto the result array (this differs from the C<DBI>).  Or to specify which columns to include in the result use the 'Columns' attribute in C<%attr> - see L<DBI-E<gt>selectcol_arrayref|DBI/"selectcol_arrayref">.
 
 =cut
 
@@ -501,21 +609,30 @@ sub col_arrayref {
     my $me = shift;
     my $attr = shift;
     $me->_sql($me->sql, $me->_bind_params_select($me->{build_data}));
-    my $sth = $me->sth->{Active} ? $me->rdbh->prepare($me->{sql}) : $me->{sth};
-    return unless $sth and $sth->execute($me->_bind_params_select($me->{build_data}));
-    my @columns = ($attr->{Columns}) ? @{$attr->{Columns}} : (1 .. $sth->{NUM_OF_FIELDS});
-    my @ary;
-    while (my $ref = $sth->fetch) {
-        push @ary, @$ref;
+    my $sth = $me->rdbh->prepare($me->{sql}, $attr) or return;
+    unless (defined $attr->{Columns}) {
+        # Some drivers don't provide $sth->{NUM_OF_FIELDS} until after execute is called
+        if ($sth->{NUM_OF_FIELDS}) {
+            $attr->{Columns} = [1 .. $sth->{NUM_OF_FIELDS}];
+        } else {
+            $sth->execute($me->_bind_params_select($me->{build_data})) or return;
+            my @col;
+            if (my $max = $attr->{MaxRows}) {
+                push @col, @$_ while 0 < $max-- and $_ = $sth->fetch;
+            } else {
+                push @col, @$_ while $_ = $sth->fetch;
+            }
+            return \@col;
+        }
     }
-    \@ary;
+    return $me->rdbh->selectcol_arrayref($sth, $attr, $me->_bind_params_select($me->{build_data}));
 }
 
-=head2 fetch
+=head3 C<fetch>
 
   my $row = $query->fetch;
 
-Fetch the next row from the query. This will run/rerun the query if needed.
+Fetch the next row from the query.  This will run/rerun the query if needed.
 
 Returns a L<DBIx::DBO::Row|DBIx::DBO::Row> object or undefined if there are no more rows.
 
@@ -542,7 +659,7 @@ sub fetch {
     ($$row->{array} = $me->{sth}->fetch) ? $me->{Row} : undef %$row;
 }
 
-=head2 row
+=head3 C<row>
 
   my $row = $query->row;
 
@@ -556,7 +673,7 @@ sub row {
     $me->{Row} ||= $me->{DBO}->row($me);
 }
 
-=head2 run
+=head3 C<run>
 
   $query->run;
 
@@ -579,7 +696,8 @@ sub run {
 sub _execute {
     my $me = shift;
     $me->_sql($me->sql, $me->_bind_params_select($me->{build_data}));
-    $me->sth->execute($me->_bind_params_select($me->{build_data}));
+    $me->sth or return;
+    $me->{sth}->execute($me->_bind_params_select($me->{build_data}));
 }
 
 sub _bind_cols_to_hash {
@@ -594,7 +712,7 @@ sub _bind_cols_to_hash {
     }
 }
 
-=head2 rows
+=head3 C<rows>
 
   my $row_count = $query->rows;
 
@@ -616,7 +734,7 @@ sub rows {
     $me->{Row_Count};
 }
 
-=head2 count_rows
+=head3 C<count_rows>
 
   my $row_count = $query->count_rows;
 
@@ -642,7 +760,7 @@ sub count_rows {
     return $count;
 }
 
-=head2 found_rows
+=head3 C<found_rows>
 
   $query->config(CalcFoundRows => 1); # Only applicable to MySQL
   my $total_rows = $query->found_rows;
@@ -663,36 +781,7 @@ sub found_rows {
     $me->{Found_Rows};
 }
 
-=head2 sth
-
-  my $sth = $query->sth;
-
-Reutrns the C<DBI> statement handle from the query.
-This will run/rerun the query if needed.
-
-=cut
-
-sub sth {
-    my $me = shift;
-    # Ensure the sql is rebuilt if needed
-    my $sql = $me->sql;
-    $me->{sth} ||= $me->rdbh->prepare($sql);
-}
-
-=head2 finish
-
-  $query->finish;
-
-Calls L<DBI-E<gt>finish|DBI/"finish"> on the statement handle, if it's active.
-
-=cut
-
-sub finish {
-    my $me = shift;
-    $me->{sth}->finish if $me->{sth} and $me->{sth}{Active};
-}
-
-=head2 sql
+=head3 C<sql>
 
   my $sql = $query->sql;
 
@@ -727,22 +816,71 @@ sub _build_sql {
     $me->{sql} = $me->_build_sql_select($me->{build_data});
 }
 
-=head2 config
+=head3 C<sth>
 
-  $query_setting = $dbo->config($option)
-  $dbo->config($option => $query_setting)
+  my $sth = $query->sth;
 
-Get or set this C<DBIx::DBO::Query> config settings.
-When setting an option, the previous value is returned.
+Reutrns the C<DBI> statement handle from the query.
+This will run/rerun the query if needed.
+
+=cut
+
+sub sth {
+    my $me = shift;
+    # Ensure the sql is rebuilt if needed
+    my $sql = $me->sql;
+    $me->{sth} ||= $me->rdbh->prepare($sql);
+}
+
+=head3 C<finish>
+
+  $query->finish;
+
+Calls L<DBI-E<gt>finish|DBI/"finish"> on the statement handle, if it's active.
+
+=cut
+
+sub finish {
+    my $me = shift;
+    $me->{sth}->finish if $me->{sth} and $me->{sth}{Active};
+}
+
+=head2 Common Methods
+
+These methods are accessible from all DBIx::DBO* objects.
+
+=head3 C<dbh>
+
+The I<read-write> C<DBI> handle.
+
+=head3 C<rdbh>
+
+The I<read-only> C<DBI> handle, or if there is no I<read-only> connection, the I<read-write> C<DBI> handle.
+
+=head3 C<do>
+
+  $dbo->do($statement)         or die $dbo->dbh->errstr;
+  $dbo->do($statement, \%attr) or die $dbo->dbh->errstr;
+  $dbo->do($statement, \%attr, @bind_values) or die ...
+
+This provides access to L<DBI-E<gt>do|DBI/"do"> method.  It defaults to using the I<read-write> C<DBI> handle.
+
+=head3 C<config>
+
+  $query_setting = $dbo->config($option);
+  $dbo->config($option => $query_setting);
+
+Get or set this C<Query> object's config settings.  When setting an option, the previous value is returned.  When getting an option's value, if the value is undefined, the L<DBIx::DBO|DBIx::DBO>'s value is returned.
+
+See L<DBIx::DBO/available_config_options>.
 
 =cut
 
 sub config {
     my $me = shift;
     my $opt = shift;
-    my $val = defined $me->{Config}{$opt} ? $me->{Config}{$opt} : $me->{DBO}->config($opt);
-    $me->{Config}{$opt} = shift if @_;
-    return $val;
+    return $me->_set_config($me->{Config} ||= {}, $opt, shift) if @_;
+    return defined $me->{Config}{$opt} ? $me->{Config}{$opt} : $me->{DBO}->config($opt);
 }
 
 sub DESTROY {
@@ -750,3 +888,31 @@ sub DESTROY {
 }
 
 1;
+
+__END__
+
+=head1 TODO LIST
+
+=over
+
+=item *
+
+Better explanation of how to construct complex queries.  This module is currently still in development (including the documentation), but I will be adding to/completing it in the near future.
+
+=item *
+
+Add C<DISTINCT> and C<COLLATE> functionality.
+
+=item *
+
+Possibly add C<open_join_on_bracket> for adding parentheses in the JOIN ON clause, but I'm not sure if this would ever be useful.
+
+=back
+
+=head1 SEE ALSO
+
+L<DBIx::DBO>
+
+
+=cut
+
