@@ -31,36 +31,27 @@ DBIx::DBO::Row - An OO interface to SQL queries and results.  Encapsulates a fet
 
 =head1 METHODS
 
-=head2 dbh
+=head3 C<new>
 
-The read-write C<DBI> handle.
+  DBIx::DBO::Row->new($dbo, $table_object);
+  DBIx::DBO::Row->new($dbo, $query_object);
 
-=head2 rdbh
-
-The read-only C<DBI> handle, or if there is no read-only connection, the read-write C<DBI> handle.
-
-=head2 do
-
-  $dbo->do($statement)         or die $dbo->dbh->errstr;
-  $dbo->do($statement, \%attr) or die $dbo->dbh->errstr;
-  $dbo->do($statement, \%attr, @bind_values) or die ...
-
-This provides access to the L<DBI-E<gt>do|DBI/"do"> method. It defaults to using the read-write C<DBI> handle.
+Create and return a new C<Row> object.
 
 =cut
 
 sub dbh { ${$_[0]}->{DBO}->dbh }
 sub rdbh { ${$_[0]}->{DBO}->rdbh }
 
-sub _new {
+sub new {
     my $proto = shift;
     my $class = ref($proto) || $proto;
     my $me = \{ DBO => shift, Parent => shift, array => undef, hash => {} };
     blessed $$me->{DBO} and $$me->{DBO}->isa('DBIx::DBO') or ouch 'Invalid DBO Object';
     ouch 'Invalid Parent Object' unless defined $$me->{Parent};
     $$me->{Parent} = $$me->{DBO}->table($$me->{Parent}) unless blessed $$me->{Parent};
-    _init($me);
-    bless $me, $class;
+    bless $me, $$me->{DBO}->_create_dbd_class($class, __PACKAGE__);
+    $me->_init;
     return wantarray ? ($me, $me->tables) : $me;
 }
 
@@ -69,7 +60,7 @@ sub _init {
     $$me->{build_data}{LimitOffset} = [1];
     if ($$me->{Parent}->isa('DBIx::DBO::Query')) {
         $$me->{Tables} = [ @{$$me->{Parent}{Tables}} ];
-        _copy_build_data($me);
+        $me->_copy_build_data;
         # We must weaken this to avoid a circular reference
         weaken $$me->{Parent};
     } elsif ($$me->{Parent}->isa('DBIx::DBO::Table')) {
@@ -94,9 +85,9 @@ sub _copy_build_data {
     }
 }
 
-=head2 tables
+=head3 C<tables>
 
-Return a list of L<DBIx::DBO::Table> objects for this row.
+Return a list of L<DBIx::DBO::Table|DBIx::DBO::Table> objects for this row.
 
 =cut
 
@@ -138,13 +129,19 @@ sub _column_idx {
     return;
 }
 
-=head2 value
+=head3 C<value>
 
-  $row->value($column)
+  $value = $row->value($column);
+  $value = $row ** $column;
 
-Return the value in this field.
+Return the value in the C<$column> field.  The C<**> method is a shortcut for the C<value> method.
 C<$column> can be a column name or a C<Column> object.
 
+Values in the C<Row> can also be obtained by using the object as an array/hash reference.
+
+  $value = $row->[2];
+  $value = $row->{some_column};
+
 =cut
 
 sub value {
@@ -160,7 +157,7 @@ sub value {
     ouch 'No such column: '.$col;
 }
 
-=head2 load
+=head3 C<load>
 
   $row->load(id => 123);
   $row->load(name => 'Bob', status => 'Employed');
@@ -209,7 +206,7 @@ sub _detach {
     undef $$me->{Parent};
 }
 
-=head2 update
+=head3 C<update>
 
   $row->update(id => 123);
   $row->update(name => 'Bob', status => 'Employed');
@@ -237,7 +234,7 @@ sub update {
     $me->do($sql, undef, $me->_bind_params_update($build_data));
 }
 
-=head2 delete
+=head3 C<delete>
 
   $row->delete;
 
@@ -277,23 +274,43 @@ sub _build_data_matching_this_row {
     return \%h;
 }
 
-=head2 config
+=head2 Common Methods
 
-  $row_setting = $dbo->config($option)
-  $dbo->config($option => $row_setting)
+These methods are accessible from all DBIx::DBO* objects.
 
-Get or set the L<DBIx::DBO::Row|DBIx::DBO::Row> config settings.
-When setting an option, the previous value is returned.
+=head3 C<dbh>
+
+The I<read-write> C<DBI> handle.
+
+=head3 C<rdbh>
+
+The I<read-only> C<DBI> handle, or if there is no I<read-only> connection, the I<read-write> C<DBI> handle.
+
+=head3 C<do>
+
+  $dbo->do($statement)         or die $dbo->dbh->errstr;
+  $dbo->do($statement, \%attr) or die $dbo->dbh->errstr;
+  $dbo->do($statement, \%attr, @bind_values) or die ...
+
+This provides access to the L<DBI-E<gt>do|DBI/"do"> method.  It defaults to using the I<read-write> C<DBI> handle.
+
+=head3 C<config>
+
+  $row_setting = $dbo->config($option);
+  $dbo->config($option => $row_setting);
+
+Get or set the C<Row> config settings.  When setting an option, the previous value is returned.  When getting an option's value, if the value is undefined, the C<Query> object (If the the C<Row> belongs to one) or L<DBIx::DBO|DBIx::DBO>'s value is returned.
+
+See L<DBIx::DBO/available_config_options>.
 
 =cut
 
 sub config {
     my $me = shift;
     my $opt = shift;
-    my $val = defined $$me->{Config}{$opt} ? $$me->{Config}{$opt} :
+    return $me->_set_config($$me->{Config} ||= {}, $opt, shift) if @_;
+    return defined $$me->{Config}{$opt} ? $$me->{Config}{$opt} :
         (defined $$me->{Parent} ? $$me->{Parent} : $$me->{DBO})->config($opt);
-    $$me->{Config}{$opt} = shift if @_;
-    return $val;
 }
 
 sub DESTROY {
@@ -301,3 +318,13 @@ sub DESTROY {
 }
 
 1;
+
+__END__
+
+=head1 SEE ALSO
+
+L<DBIx::DBO>
+
+
+=cut
+
