@@ -29,35 +29,29 @@ DBIx::DBO::Table - An OO interface to SQL queries and results.  Encapsulates a t
 
 =head1 METHODS
 
-=head2 dbh
+=head3 C<new>
 
-The read-write DBI handle.
+  DBIx::DBO::Table->new($dbo, $table);
+  DBIx::DBO::Table->new($dbo, [$schema, $table]);
+  DBIx::DBO::Table->new($dbo, $table_object);
 
-=head2 rdbh
-
-The read-only DBI handle, or if there is no read-only connection, the read-write DBI handle.
-
-=head2 do
-
-  $dbo->do($statement)         or die $dbo->dbh->errstr;
-  $dbo->do($statement, \%attr) or die $dbo->dbh->errstr;
-  $dbo->do($statement, \%attr, @bind_values) or die ...
-
-This provides access to DBI C<do> method. It defaults to using the read-write DBI handle.
+Create and return a new C<Table> object.
+Tables can be specified by their name or an arrayref of schema and table name or another C<Table> object.
 
 =cut
 
-sub _new {
+sub new {
     my ($proto, $dbo, $table) = @_;
     my $class = ref($proto) || $proto;
     blessed $dbo and $dbo->isa('DBIx::DBO') or ouch 'Invalid DBO Object';
     (my $schema, $table, $_) = $dbo->table_info($table) or ouch 'No such table: '.$table;
+    $class = $dbo->_create_dbd_class($class, __PACKAGE__);
     bless { %$_, Schema => $schema, Name => $table, DBO => $dbo, LastInsertID => undef }, $class;
 }
 
-=head2 tables
+=head3 C<tables>
 
-Return a list of L<DBIx::DBO::Table> objects, which will always be this C<Table> object.
+Return a list of C<Table> objects, which will always be this C<Table> object.
 
 =cut
 
@@ -75,12 +69,13 @@ sub _quoted_name {
     defined $me->{_quoted_name} ? $me->{_quoted_name} : ($me->{_quoted_name} = $me->_qi(@$me{qw(Schema Name)}));
 }
 
-=head2 column
+=head3 C<column>
 
-  $t->column($column_name)
-  $t ** $column_name
+  $table->column($column_name);
+  $table ** $column_name;
 
-Returns the DBO column object for this column.
+Returns a reference to a column for use with other methods.
+The C<**> method is a shortcut for the C<column> method.
 
 =cut
 
@@ -91,9 +86,26 @@ sub column {
     defined $me->{Column}{$col} ? $me->{Column}{$col} : ($me->{Column}{$col} = bless [$me, $col], 'DBIx::DBO::Column');
 }
 
-=head2 fetch_value
+=head3 C<fetch_row>
 
-  $t->fetch_value($column, %where)
+  $table->fetch_row(%where);
+
+Fetch the first matching row from the table returning it as a L<DBIx::DBO::Row|DBIx::DBO::Row> object.
+
+The C<%where> is a hash of field/value pairs.  The value can be a SCALAR ref, which will be used without quoting.
+
+  $someone = $table->fetch_row(name => \'NOT NULL', age => 21, join_date => \'CURDATE()', end_date => undef);
+
+=cut
+
+sub fetch_row {
+    my $me = shift;
+    $me->{DBO}->row($me)->load(@_);
+}
+
+=head3 C<fetch_value>
+
+  $table->fetch_value($column, %where);
 
 Fetch the first matching row from the table returning the value in one column.
 
@@ -110,9 +122,9 @@ sub fetch_value {
     return $ref && $ref->[0];
 }
 
-=head2 fetch_hash
+=head3 C<fetch_hash>
 
-  $t->fetch_hash(%where)
+  $table->fetch_hash(%where);
 
 Fetch the first matching row from the table returning it as a hashref.
 
@@ -127,22 +139,9 @@ sub fetch_hash {
     $me->rdbh->selectrow_hashref($sql, undef, @bind);
 }
 
-=head2 fetch_row
+=head3 C<fetch_column>
 
-  $t->fetch_row(%where)
-
-Fetch the first matching row from the table returning it as a Row object.
-
-=cut
-
-sub fetch_row {
-    my $me = shift;
-    $me->{DBO}->row($me)->load(@_);
-}
-
-=head2 fetch_column
-
-  $t->fetch_column($column, %where)
+  $table->fetch_column($column, %where);
 
 Fetch all matching rows from the table returning an arrayref of the values in one column.
 
@@ -158,17 +157,21 @@ sub fetch_column {
     return $me->rdbh->selectcol_arrayref($sql, undef, @bind);
 }
 
-=head2 insert
+sub _last_insert_id {
+    # Must be provided in a DBD specific method
+}
 
-  $t->insert(name => 'Richard', age => 103)
+=head3 C<insert>
 
-Insert a row into the table.
+  $table->insert(name => 'Richard', age => 103);
+
+Insert a row into the table.  Returns true on success or C<undef> on failure.
 
 =cut
 
 sub insert {
     my $me = shift;
-    ouch 'insert called without args on table '.$me->_quoted_name unless @_;
+    ouch 'Called insert() without args on table '.$me->_quoted_name unless @_;
     ouch 'Wrong number of arguments' if @_ & 1;
     my @cols;
     my @vals;
@@ -181,15 +184,15 @@ sub insert {
     $me->_sql($sql, @bind);
     my $sth = $me->dbh->prepare($sql) or return undef;
     my $rv = $sth->execute(@bind) or return undef;
-    $me->{LastInsertID} = $sth->{mysql_insertid};
+    $me->{LastInsertID} = $me->_last_insert_id($sth);
     return $rv;
 }
 
-=head2 delete
+=head3 C<delete>
 
-  $t->delete(name => 'Richard', age => 103)
+  $table->delete(name => 'Richard', age => 103);
 
-Delete all rows from the table matching the criteria.
+Delete all rows from the table matching the criteria.  Returns the number of rows deleted or C<undef> on failure.
 
 =cut
 
@@ -201,22 +204,42 @@ sub delete {
     $me->do($sql, undef, @bind);
 }
 
-=head2 config
+=head2 Common Methods
 
-  $table_setting = $dbo->config($option)
-  $dbo->config($option => $table_setting)
+These methods are accessible from all DBIx::DBO* objects.
 
-Get or set the C<DBIx::DBO::Table> config settings.
-When setting an option, the previous value is returned.
+=head3 C<dbh>
+
+The I<read-write> C<DBI> handle.
+
+=head3 C<rdbh>
+
+The I<read-only> C<DBI> handle, or if there is no I<read-only> connection, the I<read-write> C<DBI> handle.
+
+=head3 C<do>
+
+  $dbo->do($statement)         or die $dbo->dbh->errstr;
+  $dbo->do($statement, \%attr) or die $dbo->dbh->errstr;
+  $dbo->do($statement, \%attr, @bind_values) or die ...
+
+This provides access to L<DBI-E<gt>do|DBI/"do"> method.  It defaults to using the I<read-write> C<DBI> handle.
+
+=head3 C<config>
+
+  $table_setting = $table->config($option);
+  $table->config($option => $table_setting);
+
+Get or set the C<Table> config settings.  When setting an option, the previous value is returned.  When getting an option's value, if the value is undefined, the L<DBIx::DBO|DBIx::DBO>'s value is returned.
+
+See L<DBIx::DBO/available_config_options>.
 
 =cut
 
 sub config {
     my $me = shift;
     my $opt = shift;
-    my $val = defined $me->{Config}{$opt} ? $me->{Config}{$opt} : $me->{DBO}->config($opt);
-    $me->{Config}{$opt} = shift if @_;
-    return $val;
+    return $me->_set_config($me->{Config} ||= {}, $opt, shift) if @_;
+    return defined $me->{Config}{$opt} ? $me->{Config}{$opt} : $me->{DBO}->config($opt);
 }
 
 sub DESTROY {
@@ -224,3 +247,13 @@ sub DESTROY {
 }
 
 1;
+
+__END__
+
+=head1 SEE ALSO
+
+L<DBIx::DBO>
+
+
+=cut
+
